using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using CaptureTypeLib;

public class Form1 : System.Windows.Forms.Form
{
  IGraphBuilder gb = null;
  IMediaControl mc = null;
  IMediaEventEx me = null;
  IVideoWindow vw = null;
  ICaptureGraphBuilder2 cg = null;
        
  Rectangle rect; 

  //CLSID_FilterGraph (take from uuids.h DirectX include)
  Guid FgGuid = new Guid( "e436ebb3-524f-11ce-9f53-0020af0ba770" );
  //CLSID_CaptureGraphBuilder2 (take from uuids.h DirectX include)
  Guid CgGuid = new Guid( "BF87B6E1-8C27-11d0-B3F0-00AA003761C5" ); 
  //CLSID_SystemDeviceEnum
  Guid DevEnumGuid = new Guid( "62BE5D10-60EB-11d0-BD3B-00A0C911CE86" );
  //CLSID_VideoInputDeviceCategory
  Guid VidCapGuid = new Guid( "860BB310-5D01-11d0-BD3B-00A0C911CE86" );
  // taken from strmif.h
  Guid IID_IBaseFilter = new Guid( "56a86895-0ad4-11ce-b03a-0020af0ba770" );
  // needed for calls to RenderStream
  Guid PIN_CATEGORY_PREVIEW = new Guid( "fb6c4282-0353-11d1-905f-0000c0cc16ba" );
  // needed for calls to RenderStream
  Guid MEDIATYPE_Video = new Guid( "73646976-0000-0010-8000-00AA00389B71" );

  const int WM_GRAPHNOTIFY    = 0x00008001;
  const int WS_CHILD          = 0x40000000;
  const int WS_CLIPCHILDREN   = 0x02000000;
  const int WS_CLIPSIBLINGS   = 0x04000000;

  /// <summary>
  /// Required designer variable.
  /// </summary>
  private System.ComponentModel.Container components = null;

  public Form1()
  {
    //
    // Required for Windows Form Designer support
    //
    InitializeComponent();

    //
    // TODO: Add any constructor code after InitializeComponent call
    //        
  }

  /// <summary>
  /// Clean up any resources being used.
  /// </summary>
  protected override void Dispose( bool disposing )
  {
    if( disposing )
    {
      if (components != null) 
      {
        components.Dispose();
      }
    }            
    base.Dispose( disposing );
  }

		#region Windows Form Designer generated code
  /// <summary>
  /// Required method for Designer support - do not modify
  /// the contents of this method with the code editor.
  /// </summary>
  private void InitializeComponent()
  {
    // 
    // Form1
    // 
    this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
    this.ClientSize = new System.Drawing.Size(344, 246);
    this.Name = "Form1";
    this.Text = "CsPlayCap ";

  }
		#endregion

  /// <summary>
  /// The main entry point for the application.
  /// </summary>
  [STAThread]
  static void Main() 
  {
    Form1 f = new Form1();
    if( f.CaptureVideo() )
      Application.Run( f );
  }

  bool CaptureVideo() 
  { 
    bool Ok = true;
    IBaseFilter sf = null; // source filter (the capture device)
    try 
    {
      // release com object (useless here but can't hurt)
      Cleanup();

      // create the filter graph object (as a IGraphBuilder)
      Type t = Type.GetTypeFromCLSID( FgGuid );
      gb = (IGraphBuilder)Activator.CreateInstance( t );

      // create the cg object and add the filter graph to it
      t = Type.GetTypeFromCLSID( CgGuid );
      cg = (ICaptureGraphBuilder2)Activator.CreateInstance( t );

      // tell the graph builder about the filter graph
      cg.SetFiltergraph( gb );     
    
      // look for a capture device  
      if( FindDevice( out sf ) )
      {
        gb.AddFilter( sf , "Video Capture" );                  
      }
      else 
      { 
        MessageBox.Show( "Can't find capture device" );
        Cleanup();
        return false;
      }

      // build the rest of the graph
      cg.RenderStream( ref PIN_CATEGORY_PREVIEW, ref MEDIATYPE_Video, sf , null, null );

      // access different interfaces, ask runtime to notify this window
      mc = (IMediaControl)gb;
      me = (IMediaEventEx)gb;
      me.SetNotifyWindow( (int)this.Handle, WM_GRAPHNOTIFY, 0 );

      // sets the video owner and style of this window
      vw = (IVideoWindow)gb;
      vw.Owner = (int)this.Handle;
      vw.WindowStyle = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;

      // sets the window size
      OnResize( null );
      // get the ball rolling
      mc.Run();
    }
    catch( Exception ex ) { 
      Cleanup();
      MessageBox.Show("Couldn't run: " + ex.Message );
      Ok = false;
    }
    return Ok;
  }

  //
  // returns the first video capture device found in the system
  //
  bool FindDevice( out IBaseFilter srcFilter ) 
  {
    bool found = true;
    ICreateDevEnum de = null;
    srcFilter = null;
    try
    {
      // create the device enumerator (COM) object
      Type t = Type.GetTypeFromCLSID( DevEnumGuid );
      de = (ICreateDevEnum)Activator.CreateInstance( t );
      IEnumMoniker em;

      // get an enumerator for the video caputre devices
      de.CreateClassEnumerator( ref VidCapGuid, out em, 0 );
      IMoniker mon;
      uint result;

      // get a reference to the 1st device
      em.RemoteNext( 1, out mon, out result );  
      object o;
      
      // get our device ready
      mon.RemoteBindToObject( null, null, ref IID_IBaseFilter, out o ); 
      srcFilter = (IBaseFilter)o; 
    }
    catch( Exception ) 
    {
      found = false;
    }
    finally
    {
      Marshal.ReleaseComObject( de );      
    }
    return found;
  }

  void Cleanup() 
  {
    if( mc != null ) 
    {
      mc.Stop();
      mc = null;
    }
    if( me != null ) 
    {
      me.SetNotifyWindow( 0, WM_GRAPHNOTIFY, 0 );
      me = null;
    }
    if( vw != null ) 
    {
      vw.Owner = 0;
      vw = null;
    }
    if( gb != null )
      Marshal.ReleaseComObject( gb );
    if( cg != null )
      Marshal.ReleaseComObject( cg );
  }
        
  protected override void OnResize( EventArgs e ) 
  {
    if( vw != null ) 
    {                               
      rect  = this.ClientRectangle;
      vw.SetWindowPosition( rect.Left, rect.Top, rect.Width, rect.Height );
    }   
    base.OnResize( e );
  }

  protected override void WndProc( ref Message m ) 
  {
    if( m.Msg == WM_GRAPHNOTIFY )
    {
      if( me != null )
        OnGraphNotify();
      return;
    }
    base.WndProc( ref m );
  }
    
  void OnGraphNotify()
  {
    int p1, p2;
    int code;
    try 
    {
      do
      {
        if( me == null )
          return;    
        me.GetEvent( out code, out p1, out p2, 0 );   
        me.FreeEventParams( code, p1, p2 );
       }
      while( true );
    } 
    catch( Exception ){}
  }
}

